#include "Plant-component.hpp"
#include <rtt/Component.hpp>

/*
 * Using this macro, only one component may live
 * in one library *and* you may *not* link this library
 * with another component library. Use 
 * ORO_CREATE_COMPONENT_TYPE()
 * ORO_LIST_COMPONENT_TYPE(Plant)
 * In case you want to link with another library that
 * already contains components.
 *
 * If you have put your component class
 * in a namespace, don't forget to add it here too:
 */


Plant::Plant(const string & name) :
		TaskContext(name, PreOperational), position("Position"), velocity(
				"Velocity"), setpoints("SetPoints"), stamp(0), pos(0.0), vel(
				0.0) {
	this->addProperty("Stamp", stamp).doc("OS time service tick data type");
	this->addAttribute("Pos", pos);
	this->addAttribute("Vel", vel);

	this->ports()->addPort("Position", position).doc(
			"1D Position of this plant.");
	this->ports()->addPort("Velocity", velocity).doc(
			"1D Velocity of this plant.");
	this->ports()->addPort("SetPoints", setpoints).doc(
			"1D drive setpoint of this plant.");

}

bool Plant::configureHook() {
	if (!setpoints.connected() || !position.connected()) {
		log(Error) << "Refusing to configure without connected ports."
				<< endlog();
		return false;
	}

	return true;
}

bool Plant::startHook() {
	// reset timestamp
	stamp = os::TimeService::Instance()->getTicks();
	return true;
}

void Plant::updateHook() {
	// Read setpoint.
	bool ret = setpoints.read(vel);
	if (ret == false) {
		// The current blocking API always returns true.
		// assert(false);
	} else
		pos += vel * os::TimeService::Instance()->secondsSince(stamp);
	stamp = os::TimeService::Instance()->getTicks();

	position.write(pos);
	velocity.write(vel);

	// Inform thread to call updateHook again after this one.
	this->engine()->getActivity()->trigger();
}

void Plant::stopHook() {
}

void Plant::cleanupHook() {
}

ORO_CREATE_COMPONENT(Plant)
